/*
 * logging.h
 *
 * (c)2017,2020-2021 Todd Shadburn
 *
 * Licensed under the GNU GPL version 2
 *
 *
*/


#ifndef _LOGGING_H_
#define _LOGGING_H_


#include <stdint.h>
#include <stdarg.h>
#include <pthread.h>


enum logging_level_e {
	LOG_LEVEL_EMERG	= 0,
	LOG_LEVEL_ALERT,
	LOG_LEVEL_CRITICAL,
	LOG_LEVEL_ERROR,
	LOG_LEVEL_WARNING,
	LOG_LEVEL_NOTICE,
	LOG_LEVEL_INFO,
	LOG_LEVEL_DEBUG,
	_LOG_LEVEL_MAX,
};



#define LOGGING_FILEPATH_SIZE	1024
#define LOGGING_BUFFER_SIZE	8192

#define LOGGING_OPT_MUTEX		0x00000001
#define LOGGING_OPT_FLUSH_LOG_ENTRIES	0x00000002


typedef struct logging_s logging_t;
struct logging_s {
	uint32_t	options;
	int		level;
	FILE		*fp;
	char		filepath[LOGGING_FILEPATH_SIZE];
	pthread_mutex_t	mutex;
	char		buffer[LOGGING_BUFFER_SIZE];
};

logging_t *logging_open(char *filepath, uint32_t options);
int logging_close(logging_t *log);
int logging_reopen(logging_t *log, char *filepath, uint32_t options);
void logging_entry(logging_t *log, int level, char *format, ...);
void logging_entry_va(logging_t *log, int level, char *format, va_list args);
void logging_set_log_level(logging_t *log, int level);
int logging_string_to_level(char* str);

#endif


