/************************************************************************/
/* pnetbuffer.h								*/
/************************************************************************/
/*
 *  (c)2017,2020-2021 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/




#ifndef _PNETBUFFER_H_
#define _PNETBUFFER_H_

#include <stdint.h>


typedef enum {
	PNETBUFFER_RC_SUCCESS	= 0,
	PNETBUFFER_RC_NOSPACE,
	PNETBUFFER_RC_NOTENOUGH,
	PNETBUFFER_RC_MISSINGOPT,
} pnetbuffer_rc_t;


#define PNETBUFFER_OPT_STRINGS_COUNT_FIRST      0x01
#define PNETBUFFER_OPT_STRINGS_LAST_EMPTY       0x02

// Opaque prototype
typedef struct pnetbuffer_s	pnetbuffer_t;


pnetbuffer_t *pnetbuffer_init(char *buffer, uint16_t len);
void pnetbuffer_reset(pnetbuffer_t *nb);
void pnetbuffer_reset_read(pnetbuffer_t *nb);

// add functions take LSB values and writes to the buffer in MSB
uint8_t pnetbuffer_add_uint8(pnetbuffer_t *nb, uint8_t val);
uint8_t pnetbuffer_add_uint16(pnetbuffer_t *nb, uint16_t val);
uint8_t pnetbuffer_add_uint32(pnetbuffer_t *nb, uint32_t val);
uint8_t pnetbuffer_add_uint64(pnetbuffer_t *nb, uint64_t val);
uint8_t pnetbuffer_add_float(pnetbuffer_t *nb, float val);
uint8_t pnetbuffer_add_data(pnetbuffer_t *nb, char *data, uint16_t datalen);
uint8_t pnetbuffer_add_strings(pnetbuffer_t *nb, char **strings, uint8_t count, uint8_t options);


// read functions take MSB values from the buffer and return LSB values
uint8_t pnetbuffer_read_uint8(pnetbuffer_t *nb, uint8_t *val);
uint8_t pnetbuffer_read_uint16(pnetbuffer_t *nb, uint16_t *val);
uint8_t pnetbuffer_read_uint32(pnetbuffer_t *nb, uint32_t *val);
uint8_t pnetbuffer_read_uint64(pnetbuffer_t *nb, uint64_t *val);
uint8_t pnetbuffer_read_float(pnetbuffer_t *nb, float *val);
uint8_t pnetbuffer_read_data(pnetbuffer_t *nb, char *data, uint16_t datalen);
uint8_t pnetbuffer_read_strings(pnetbuffer_t *nb,
				char **array,		// array allocated by caller
				uint8_t maxlen,		// max length of array elements
				uint8_t maxcount,	// maximum array length
				uint8_t *count,		// returned count of array elements filled
				uint8_t options);	// processing options

void pnetbuffer_get_readable(pnetbuffer_t *nb, char **ptr, uint16_t *len);
uint16_t pnetbuffer_get_writable_len(pnetbuffer_t *nb);

void pnetbuffer_get_writable(pnetbuffer_t *nb, char **ptr, uint16_t *len);
uint8_t pnetbuffer_set_written(pnetbuffer_t *nb, uint16_t len);

// write functions take LSB values and writes to the buffer in MSB
// at the given offset into the buffer
uint8_t pnetbuffer_write_uint8(pnetbuffer_t *nb, uint16_t offset, uint8_t val);
uint8_t pnetbuffer_write_uint16(pnetbuffer_t *nb, uint16_t offset, uint16_t val);
uint8_t pnetbuffer_write_uint32(pnetbuffer_t *nb, uint16_t offset, uint32_t val);
uint8_t pnetbuffer_write_float(pnetbuffer_t *nb, uint16_t offset, float val);
uint8_t pnetbuffer_write_data(pnetbuffer_t *nb, uint16_t offset, char *data, uint16_t datalen);

#endif


