/************************************************************************/
/* protocols.h                                                         */
/************************************************************************/
/*
 *  (c)2021 Todd Shadburn <tshadburn@opencomputingsolutions.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/************************************************************************/




#ifndef _PROTOCOLS_H_
#define _PROTOCOLS_H_

#include <stdint.h>
#include <pnetbuffer.h>


typedef struct _ipv4net_s ipv4net_t;
struct _ipv4net_s {
	uint32_t	net;
	uint32_t	mask;
};

#define	IPV4_IN_NET(x,y)	((x&y->mask) == y->net ? 1 : 0)


#define COMPARE_ADDR_SRC	0x00000001
#define COMPARE_ADDR_DEST	0x00000002
#define COMPARE_PORT_SRC	0x00000004
#define COMPARE_PORT_DEST	0x00000008

#define COMPARE_ACTION_ALLOW	0x01
#define COMPARE_ACTION_DENY	0x03

typedef struct _compare_s compare_t;
struct _compare_s {
	uint16_t	protocol;
	uint16_t	action;
	uint32_t	compare;
	ipv4net_t	src_addr;
	ipv4net_t	dest_addr;
	uint16_t	src_port;
	uint16_t	dest_port;
};


uint8_t	pnetbuffer_read_compare(pnetbuffer_t *nb, compare_t *compare);
uint8_t	pnetbuffer_write_compare(pnetbuffer_t *nb, compare_t *compare);


uint8_t pnetbuffer_read_geneve(pnetbuffer_t *nb,
				uint16_t *ethertype,
				uint32_t *vni,
				char *option_data,
				uint16_t *option_len);

uint8_t pnetbuffer_read_ethernet2(pnetbuffer_t *nb,
				char src_mac[6],
				char dest_mac[6],
				uint16_t *type);

uint8_t pnetbuffer_read_ipv4(pnetbuffer_t *nb,
				uint32_t *src_ip,
				uint32_t *dest_ip,
				uint16_t *protocol,
				uint16_t *length,
				uint16_t *checksum);

uint8_t pnetbuffer_read_icmp(pnetbuffer_t *nb,
				uint8_t *type,
				uint8_t *code,
				uint16_t *checksum);

uint8_t pnetbuffer_read_udp(pnetbuffer_t *nb,
				uint16_t *src_port,
				uint16_t *dest_port,
				uint16_t *checksum,
				uint16_t *length);

uint8_t pnetbuffer_read_tcp(pnetbuffer_t *nb,
				uint16_t *src_port,
				uint16_t *dest_port,
				uint32_t *seq_num,
				uint32_t *ack_num,
				uint16_t *flags,
				uint16_t *window,
				uint16_t *checksum,
				uint16_t *urgent,
				char *option_data,
				uint16_t *option_len,
				uint16_t *length);



#endif


